import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flamingo/flamingo.dart';

class User extends Document<User> {
  User({
    String id,
    DocumentSnapshot snapshot,
    Map<String, dynamic> values,
  }): super(id: id, snapshot: snapshot, values: values) {
    settingsA = Collection(this, 'settingsA');
    settingsB = Collection(this, 'settingsB');
    likeCounter = Counter(this, 'likeShards', numShards);
  }

  /// Document
  String uid;
  String name;
  int age;
  List<String> memos;

  /// DistributedCounter
  int numShards = 10;
  Counter likeCounter;

  /// SubCollection
  Collection settingsA;
  Collection settingsB;

  /// For save data
  @override
  Map<String, dynamic> toData() {
    final data = <String, dynamic>{};
    write(data, 'uid', uid);
    write(data, 'name', name);
    write(data, 'age', age);
    write(data, 'memos', memos);
    return data;
  }

  /// For load data
  @override
  void fromData(Map<String, dynamic> data) {
    this..uid = valueFromKey<String>(data, 'uid')
      ..name = valueFromKey<String>(data, 'name')
      ..age = valueFromKey<int>(data, 'age')
      ..numShards = valueFromKey<int>(data, 'numShards')
      ..memos = valueListFromKey<String>(data, 'memos');
  }

  void log() {
    print('$id $uid $name $age $numShards $createdAt $updatedAt');
    print('$memos');
    print('$settingsA $settingsB');
  }

}
